package RPM::Source::Transformation::Writer::Plugin::GearUpdater;

use strict;
use warnings;

use File::Spec;
use File::Path  qw(make_path remove_tree);
use Cwd qw/cwd abs_path/;
use Gear::Remotes::Tags 0.003;
use RPM::Source::Transformation::EarlyResource::GearRules;
use RPM::Source::Transformation::Factory::AddGearRemotes;
use RPM::Source::Transformation::Resource::FlagLoader;
use RPM::Source::Transformation::Factory::BundleImport::DownloadInput;
use RPM::Source::Transformation::Component::GirarNMU::Options;
use RPM::Source::Transformation::Writer::GirarNMU;
use RPM::Source::Transformation::Component;
our @ISA=qw/RPM::Source::Transformation::Component/;
$RPM::Source::Transformation::Writer::GirarNMU::GEAR_WRITER=__PACKAGE__;

our $VERSION=0.003;
our $verbose=0;

sub process_options {
    $RPM::Source::Transformation::Factory::BundleImport::DownloadInput::download_gear_cachedir=$RPM::Source::Transformation::Component::GirarNMU::Options::outgeardir.'/downloads';
}

sub write {
    my (undef,$spec, undef,$converter_opt,$resourceobj)=@_;
    my $rpmname=$converter_opt->{-input_handler_arg_input_name};
    my $curdir=cwd;
    my $abs_outgeardir=File::Spec->rel2abs( $RPM::Source::Transformation::Component::GirarNMU::Options::outgeardir );
    my $specfilesdir=$abs_outgeardir.'/specfiles/';
    my $specoverride=$specfilesdir.$rpmname.'.spec';
    make_path($specfilesdir);
    &__write_spec_as_specfile($spec,$specoverride);

    my $infile=$converter_opt->{-opt_bundle_source_file};
    $infile//=$converter_opt->{-opt_bundle_input_file};
    my $file=abs_path($infile);
    my $newver=$converter_opt->{-opt_bundle_version};

    my $move_to_subdir;
    my $geardir=$spec->{GEARDIR};
    chdir $geardir;
    my $rules=$resourceobj->get_cached_gear_rules();
    my $specfile=$rules->get_spec();
    if (not $rules->has_external_commits()) {
	&gear_write_tarball_update($file, $newver, $specoverride);
    } else {
	if (0==system(qw/gear-remotes-restore/)) {
	    &__run(qw/gear-remotes-fetch/);
	    my $tag=&Gear::Remotes::Tags::find_local_tag_for_version($newver);
	    if (defined $tag) {
		&gear_write_tag_update($tag, $newver, $specoverride);
	    } else {
		&gear_write_manualmerge_tagupdate($specfile, $specoverride);
		&gear_report_unknown_repo($rpmname, $abs_outgeardir, 'NOTAG');
	    }
	} else {
	    warn "${rpmname}.git has external commits.";
	    &gear_write_tarball_update($file, $newver, $specoverride);
	    &gear_report_unknown_repo($rpmname, $abs_outgeardir, 'EXTERNAL');
	}
    }
    chdir $curdir;
    $spec->cleanup();
    # ok
    return 1;
}

sub gear_write_manualmerge_tagupdate {
    my ($specfile, $specoverride)=@_;
    &__writefile('manualmerge-tagupdate.sh',q{#!/bin/sh
echo "merge tag first!!!"
rm -f }.$specfile.q{
cp -a }.$specoverride.' '.$specfile.q{
git add }.$specfile.q{
gear-update-tag -ac
});
    &__run(qw/chmod 755 manualmerge-tagupdate.sh/);
}

sub gear_write_tag_update {
    my ($tag, $newver, $specoverride)=@_;
    &__writefile('tagupdate.sh', 'gear-uupdate --upstream-version '.$newver.' --tag '.$tag.' --override-spec '.$specoverride."\n");
    &__run(qw/chmod 755 tagupdate.sh/);
}

sub gear_write_tarball_update {
    my ($file, $newver, $specoverride)=@_;
    &__writefile('update.sh',"gear-uupdate --upstream-version $newver --override-spec $specoverride $file\n");
    &__run(qw/chmod 755 update.sh/);
}

sub gear_report_unknown_repo {
    my ($rpmname,$abs_outgeardir,$destname)=@_;
    my $destdir=$abs_outgeardir.'/'.$destname;
    make_path($destdir);
    &__run('mv', $abs_outgeardir.'/'.${rpmname}.'.git', $destdir);
}

sub __write_spec_as_specfile {
    my ($spec,$specfile)=@_;
    open (my $fn, '>', $specfile) or die "$!: can't rewrite specfile $specfile";
    print $fn $spec->get_spec();
    close ($fn) or die "$!: can't close specfile $specfile";
}

sub __writefile {
    my ($file,@data)=@_;
    open (my $fn, '>', $file) or die "$!: can't write to $file";
    print $fn @data;
    close ($fn) or die "$!: can't close $file";
}

sub __run {
    print STDERR "running: ", join(' ',@_),"\n";
    system(@_)==0 or die "command failed: ".join(' ',@_);
}

1;
__END__
